/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.metal;

import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.ITileEntityProvider;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.inventory.InventoryHelper;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import mcp.MethodsReturnNonnullByDefault;
import net.dries007.tfc.objects.Metal;
import net.dries007.tfc.objects.blocks.BlocksTFC;
import net.dries007.tfc.objects.items.metal.ItemMetal;
import net.dries007.tfc.objects.te.TEIngotPile;
import net.dries007.tfc.util.Helpers;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public class BlockIngotPile extends Block implements ITileEntityProvider
{
    public BlockIngotPile()
    {
        super(Material.field_151573_f);

        func_149711_c(3.0F);
        func_149752_b(10.0F);
        setHarvestLevel("pickaxe", 0);
    }

    @Nullable
    @Override
    public TileEntity func_149915_a(World worldIn, int meta)
    {
        return new TEIngotPile();
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149686_d(IBlockState state)
    {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Override
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        TEIngotPile te = Helpers.getTE(source, pos, TEIngotPile.class);
        double y = te != null ? 0.125 * (te.getCount() / 8) : 1;
        return new AxisAlignedBB(0d, 0d, 0d, 1d, y, 1d);
    }

    @Override
    @SuppressWarnings("deprecation")
    public BlockFaceShape func_193383_a(IBlockAccess worldIn, IBlockState state, BlockPos pos, EnumFacing face)
    {
        return BlockFaceShape.UNDEFINED;
    }

    @SuppressWarnings("deprecation")
    @Override
    public AxisAlignedBB func_180646_a(IBlockState blockState, IBlockAccess worldIn, BlockPos pos)
    {
        TEIngotPile te = Helpers.getTE(worldIn, pos, TEIngotPile.class);
        double y = te != null ? 0.125 * (te.getCount() / 8) : 1;
        return new AxisAlignedBB(0d, 0d, 0d, 1d, y, 1d);
    }

    @SideOnly(Side.CLIENT)
    @SuppressWarnings("deprecation")
    @Override
    public AxisAlignedBB func_180640_a(IBlockState state, World worldIn, BlockPos pos)
    {
        TEIngotPile te = Helpers.getTE(worldIn, pos, TEIngotPile.class);
        double y = te != null ? 0.125 * (te.getCount() / 8) : 1;
        return new AxisAlignedBB(0d, 0d, 0d, 1d, y, 1d);
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149662_c(IBlockState state)
    {
        return false;
    }

    @Override
    @SuppressWarnings("deprecation")
    public void func_189540_a(IBlockState state, World worldIn, BlockPos pos, Block blockIn, BlockPos fromPos)
    {
        if (!collapseDown(worldIn, pos) && !worldIn.isSideSolid(pos.func_177977_b(), EnumFacing.UP))
        {
            worldIn.func_175698_g(pos);
        }
    }

    @Override
    public void func_180663_b(World worldIn, BlockPos pos, IBlockState state)
    {
        TEIngotPile te = Helpers.getTE(worldIn, pos, TEIngotPile.class);
        if (te != null) te.onBreakBlock();
        super.func_180663_b(worldIn, pos, state);
    }

    @Override
    public boolean func_180639_a(World worldIn, BlockPos pos, IBlockState state, EntityPlayer playerIn, EnumHand hand, EnumFacing facing, float hitX, float hitY, float hitZ)
    {
        if (playerIn.func_70093_af()) return true;

        TEIngotPile te = Helpers.getTE(worldIn, pos, TEIngotPile.class);
        if (te == null) return true;

        BlockPos posTop = pos;
        IBlockState stateTop;
        do
        {
            posTop = posTop.func_177984_a();
            stateTop = worldIn.func_180495_p(posTop);
            //noinspection ConstantConditions
            if (stateTop.func_177230_c() != BlocksTFC.INGOT_PILE)
            {
                te.setCount(te.getCount() - 1);
                if (!worldIn.field_72995_K)
                {
                    if (te.getCount() <= 0)
                    {
                        worldIn.func_175656_a(pos, Blocks.field_150350_a.func_176223_P());
                    }
                    InventoryHelper.func_180173_a(worldIn, posTop.func_177958_n(), posTop.func_177977_b().func_177956_o() + 0.125 * (te.getCount() / 8 + 2), posTop.func_177952_p(), new ItemStack(ItemMetal.get(te.getMetal(), Metal.ItemType.INGOT)));

                }
                return true;
            }
            else
            {
                te = Helpers.getTE(worldIn, posTop, TEIngotPile.class);
                if (te != null)
                {
                    if (te.getCount() < 64)
                    {
                        te.setCount(te.getCount() - 1);
                        if (!worldIn.field_72995_K)
                        {
                            if (te.getCount() <= 0)
                            {
                                worldIn.func_175656_a(pos, Blocks.field_150350_a.func_176223_P());
                            }
                            InventoryHelper.func_180173_a(worldIn, posTop.func_177958_n(), posTop.func_177956_o() + 0.125 * (te.getCount() / 8 + 2), posTop.func_177952_p(), new ItemStack(ItemMetal.get(te.getMetal(), Metal.ItemType.INGOT)));
                        }
                        return true;
                    }
                }
            }
        } while (posTop.func_177956_o() <= 256);
        return false;
    }

    @Override
    public ItemStack getPickBlock(IBlockState state, RayTraceResult target, World world, BlockPos pos, EntityPlayer player)
    {
        TEIngotPile te = Helpers.getTE(world, pos, TEIngotPile.class);
        return new ItemStack(ItemMetal.get((te != null ? te.getMetal() : Metal.UNKNOWN), Metal.ItemType.INGOT));
    }

    private boolean collapseDown(World world, BlockPos pos)
    {
        IBlockState stateDown = world.func_180495_p(pos.func_177977_b());
        //noinspection ConstantConditions
        if (stateDown.func_177230_c() == BlocksTFC.INGOT_PILE)
        {

            TEIngotPile te = Helpers.getTE(world, pos.func_177977_b(), TEIngotPile.class);
            TEIngotPile teUp = Helpers.getTE(world, pos, TEIngotPile.class);
            if (te != null && teUp != null && te.getCount() < 64)
            {
                if (te.getCount() + teUp.getCount() <= 64)
                {
                    te.setCount(te.getCount() + teUp.getCount());
                    world.func_175698_g(pos);
                }
                else
                {
                    te.setCount(64);
                    teUp.setCount(te.getCount() + teUp.getCount() - 64);
                }
            }
            return true;
        }
        return false;
    }
}
