/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.items.metal;

import javax.annotation.ParametersAreNonnullByDefault;

import com.google.common.collect.Multimap;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.SharedMonsterAttributes;
import net.minecraft.entity.ai.attributes.AttributeModifier;
import net.minecraft.inventory.EntityEquipmentSlot;
import net.minecraft.item.ItemStack;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import mcp.MethodsReturnNonnullByDefault;
import net.dries007.tfc.objects.Metal;

import net.minecraft.item.Item.ToolMaterial;

@ParametersAreNonnullByDefault
@MethodsReturnNonnullByDefault
public class ItemMetalTool extends ItemMetal
{
    public final ToolMaterial material;
    private final float efficiency;
    private final double attackDamage;
    private final int areaOfAttack; // todo: implement
    private final float attackSpeed;

    public ItemMetalTool(Metal metal, Metal.ItemType type)
    {
        super(metal, type);
        if (metal.toolMetal == null) throw new IllegalArgumentException("You can't make tools out of non tool metals.");
        material = metal.toolMetal;
        float typeDamage;
        switch (type)
        {
            case PICK:
                setHarvestLevel("pickaxe", metal.toolMetal.func_77996_d());
                typeDamage = 1.2f; // todo: use some central spot for this (config maybe?) and make the rock equivalents use the same numbers.
                areaOfAttack = 1;
                attackSpeed = -2.8f;
                break;
            case SHOVEL:
                setHarvestLevel("shovel", metal.toolMetal.func_77996_d());
                typeDamage = 1.3f;
                areaOfAttack = 1;
                attackSpeed = -3f;
                break;
            case AXE:
                setHarvestLevel("axe", metal.toolMetal.func_77996_d());
                typeDamage = 1.5f;
                areaOfAttack = 1;
                attackSpeed = -3f;
                break;
            case HOE:
                setHarvestLevel("hoe", metal.toolMetal.func_77996_d());
                typeDamage = 0.7f;
                areaOfAttack = 1;
                attackSpeed = -3;
                break;
            case CHISEL:
                setHarvestLevel("chisel", metal.toolMetal.func_77996_d());
                typeDamage = 0.7f;
                areaOfAttack = 1;
                attackSpeed = 0;
                break;
            case SAW:
                setHarvestLevel("saw", metal.toolMetal.func_77996_d());
                typeDamage = 0.5f;
                areaOfAttack = 1;
                attackSpeed = -1;
                break;
            case PROPICK:
                setHarvestLevel("pickaxe", metal.toolMetal.func_77996_d());
                typeDamage = 1f;
                areaOfAttack = 1;
                attackSpeed = -3.5f;
                break;
            case SCYTHE:
                setHarvestLevel("scythe", metal.toolMetal.func_77996_d());
                typeDamage = 1.5f;
                areaOfAttack = 3;
                attackSpeed = -3.5f;
                break;
            case KNIFE:
                setHarvestLevel("knife", metal.toolMetal.func_77996_d());
                typeDamage = 0.5f;
                areaOfAttack = 1;
                attackSpeed = 3f;
                break;
            case HAMMER:
                setHarvestLevel("hammer", metal.toolMetal.func_77996_d());
                typeDamage = 2f;
                areaOfAttack = 1;
                attackSpeed = -3.5f;
                break;
            case SWORD:
                typeDamage = 1f;
                areaOfAttack = 1;
                attackSpeed = -0.75f;
                break;
            case MACE:
                typeDamage = 1.1f;
                areaOfAttack = 1;
                attackSpeed = -1;
                break;
            case JAVELIN:
                typeDamage = 1f;
                areaOfAttack = 1;
                attackSpeed = -1;
                break;
            default:
                throw new IllegalArgumentException("Tool from non tool type.");
        }

        func_77625_d(1);
        func_77656_e(material.func_77997_a());
        efficiency = material.func_77998_b();
        attackDamage = typeDamage * material.func_78000_c();
    }

    @Override
    public float func_150893_a(ItemStack stack, IBlockState state)
    {
        Material material = state.func_185904_a();
        switch (type)
        {
            case AXE:
                if (material == Material.field_151575_d || material == Material.field_151585_k || material == Material.field_151582_l)
                    return efficiency;
                break;
            case PICK:
                if (material == Material.field_151573_f || material == Material.field_151574_g || material == Material.field_151576_e)
                    return efficiency;
                break;
            case SHOVEL:
                if (material == Material.field_151597_y || material == Material.field_151596_z) return efficiency;
            case SCYTHE:
                if (material == Material.field_151585_k || material == Material.field_151582_l || material == Material.field_151584_j)
                    return efficiency;
        }
        for (String type : getToolClasses(stack))
        {
            if (state.func_177230_c().isToolEffective(type, state)) return efficiency;
        }
        return super.func_150893_a(stack, state);
    }

    @Override
    public boolean func_77644_a(ItemStack stack, EntityLivingBase target, EntityLivingBase attacker)
    {
        switch (type)
        {
            case PROPICK:
            case SAW:
                stack.func_77972_a(4, attacker);
                break;
            case HOE:
            case CHISEL:
                stack.func_77972_a(3, attacker);
                break;
            case PICK:
            case SHOVEL:
            case AXE:
            case SCYTHE:
                stack.func_77972_a(2, attacker);
                break;
            case SWORD:
            case MACE:
            case JAVELIN:
            case HAMMER:
            case KNIFE:
                stack.func_77972_a(1, attacker);
                break;
        }
        return true;
    }

    @Override
    public boolean func_179218_a(ItemStack stack, World worldIn, IBlockState state, BlockPos pos, EntityLivingBase entityLiving)
    {
        if (!worldIn.field_72995_K && (double) state.func_185887_b(worldIn, pos) != 0.0D)
        {
            stack.func_77972_a(1, entityLiving);
        }
        return true;
    }

    @Override
    public boolean func_150897_b(IBlockState blockIn)
    {
        return super.func_150897_b(blockIn);
    }

    @Override
    @SideOnly(Side.CLIENT)
    public boolean func_77662_d()
    {
        return true;
    }

    @Override
    public int func_77619_b()
    {
        return material.func_77995_e();
    }

    @Override
    public Multimap<String, AttributeModifier> getAttributeModifiers(EntityEquipmentSlot slot, ItemStack stack)
    {
        Multimap<String, AttributeModifier> multimap = super.getAttributeModifiers(slot, stack);
        if (slot == EntityEquipmentSlot.MAINHAND)
        {
            multimap.put(SharedMonsterAttributes.field_111264_e.func_111108_a(), new AttributeModifier(field_111210_e, "Tool modifier", attackDamage, 0));
            multimap.put(SharedMonsterAttributes.field_188790_f.func_111108_a(), new AttributeModifier(field_185050_h, "Tool modifier", attackSpeed, 0));
        }
        return multimap;
    }

    @Override
    public boolean canStack(ItemStack stack)
    {
        return false;
    }
}
